package joc.arena;

import joc.arena.regles.Bestiari;
import joc.arena.regles.Combat;
import joc.arena.regles.Lluitador;

import joc.arena.interficie.EntradaTeclat;
import joc.arena.interficie.SortidaPantalla;

public class JocArena {

  public static final int MAX_COMBAT = 2;

  private EntradaTeclat entrada = new EntradaTeclat();
  private SortidaPantalla sortida = new SortidaPantalla();
  private Lluitador lluitador = new Lluitador();
  private Combat combat = new Combat();
  private Bestiari bestiari = new Bestiari();

  public static void main(String[] args) {
    JocArena programa = new JocArena();
    programa.inici();
  }

  public void inici() {

    //PART RECURSIVA
    //bestiari.ordenarAdversaris();

    sortida.mostarBenvinguda();
    int[] jugador = bestiari.generarJugador();

    int numCombat = 0;
    boolean jugar = true;
    
    while (jugar) {
      numCombat++;

      //Abans de cada combat es restaura al jugador
      lluitador.restaurar(jugador);

      //Inici d'un combat
      System.out.println("*** COMBAT " + numCombat);
      System.out.print("Estat actual del jugador: ");
      sortida.mostrarLluitador(jugador);
      System.out.println("**************************");

      //S'obte l'adversari
      int[] adversari = entrada.triarAdversari(lluitador.llegirNivell(jugador));

      //Combat
      combatre(jugador, adversari);

      //Fi
      jugar = fiCombat(jugador, adversari);
      if (numCombat == MAX_COMBAT) {
        System.out.println("Has sobreviscut a tots els combats. Enhorabona!!");
      }
      
    }
    System.out.print("Estat final del jugador: ");
    sortida.mostrarLluitador(jugador);
  }

  /** Resol totes les rondes d'un combat
   *
   * @param jugador Estat del jugador
   * @param adversari Estat de l'adversari
   */
  public void combatre(int[] jugador, int[] adversari) {

    boolean combatre = true;
    int numRonda = 0;
    while (combatre) {
      numRonda++;
      if (numRonda%5 == 0) {
        //Les rondes multiples de cinc es restaura l'atac i defensa
        lluitador.restaurar(jugador);
        lluitador.restaurar(adversari);
      }
      System.out.println("--- RONDA " + numRonda);
      sortida.mostrarVersus(jugador, adversari);
      System.out.println("--------------------------");
      int accioJug = entrada.preguntarEstrategia();
      int accioAdv = lluitador.triarEstrategiaAtzar(adversari);
      System.out.print("Has triat " + combat.estrategiaAText(accioJug));
      System.out.println(" i el teu enemic " + combat.estrategiaAText(accioAdv));
      combat.resoldreEstrategies(jugador, accioJug, adversari, accioAdv);
      if (lluitador.esMort(jugador)||lluitador.esMort(adversari)) {
        combatre = false;
      }
    }
  }

  /** Resol la fi del combat.
   *
   * @param jugador Estat del jugador
   * @param adversari Estat de l'adversari
   * @returns Si el jugador ha de seguir jugant (true) o no (false)
   */
  public boolean fiCombat(int[] jugador, int[] adversari) {
    if (lluitador.esMort(jugador)) {
      //Has perdut (Nota: tambe inclou el cas que tots dos moren alhora)
      System.out.println("Has estat derrotat... :-(");
      return false;
    }
    System.out.println("Has guanyat el combat :-)");
    boolean pujarNivell = lluitador.atorgarPunts(jugador, adversari);
    if (pujarNivell) {
      System.out.println("Has pujat de nivell!!");
      lluitador.pujarNivell(jugador);
    }
    return true;
  }

}
